package com.holmos.cache;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Watsen 平台缓存管理类，这是一个单例类，用来管理系统中的缓存信息
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * 
 * */
public final class HolmosCacheManager {
	private ArrayList<String> subCacheClassNames=new ArrayList<String>();
	private HolmosCacheManager(){
		caches=new HashMap<String, HolmosCache>();
	}
	
	private volatile static HolmosCacheManager cacheManager=null;
	
	public static HolmosCacheManager getInstance(){
		if(null==cacheManager){
			synchronized (cacheManager) {
				if(null==cacheManager)
					cacheManager=new HolmosCacheManager();
			}
		}
		return cacheManager;
	}
	
	private HashMap<String, HolmosCache> caches;
	
	/**
	 * 获得系统里面所有的缓存对象
	 * 
	 * @return 所有缓存对象
	 * */
	public ArrayList<HolmosCache> getAllCaches(){
		return (ArrayList<HolmosCache>) caches.values();
	}
	
	/**
	 * 增加缓存对象，不能为null
	 * */
	public void addCache(HolmosCache cache){
		if(null!=cache){
			this.caches.put(cache.getCacheName(), cache);
		}
	}
	
	/**
	 * 获得名字为name的cache
	 * 
	 * */
	public HolmosCache getCache(String name){
		return caches.get(name);
	}
	
	/**移除名字为name的缓存*/
	public void removeCache(String name){
		caches.remove(name);
	}
	/**
	 * 增加HolmosCache子类的名称
	 * */
	public void addSubClassName(String className){
		if(!this.subCacheClassNames.contains(className))
			this.subCacheClassNames.add(className);
	}
	/**
	 * 删除HolmosCache子类名
	 * */
	public void removeSubClassName(String subClassName){
		this.subCacheClassNames.remove(subClassName);
	}
	public ArrayList<String>getAllSubCacheNames(){
		return subCacheClassNames;
	}
}
