package com.holmos.cache.config.parser;

import java.util.ArrayList;

import com.holmos.cache.config.HolmosCacheConfig;
import com.holmos.cache.constvalue.CacheConstValue;
import com.holmos.cache.exception.HolmosCacheException;
import com.holmos.cache.utils.CachePropertiesUtils;

/**
 * Watsen 平台缓存系统Properties格式配置文件解析器
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * 
 * */
public class PropertiesParser extends AbstractParser{
	protected ArrayList<String> cacheNames=new ArrayList<String>();
	public PropertiesParser(){
		super();
		cacheNames=(ArrayList<String>) CachePropertiesUtils.getValueList(CacheConstValue.PROCACHECONF, name);
	}
	@Override
	public HolmosCacheConfig parser(String cacheName) {
		if(!cacheNames.contains(cacheName))
			throw new HolmosCacheException("您要获取的名字为"+cacheName+"的缓存没有配置!请检查配置文件!");
		HolmosCacheConfig config=configs.get(cacheName);
		if(null==config){
			config=new HolmosCacheConfig();
			config.setName(cacheName);
			config.setMaxSize(CachePropertiesUtils.getLong(CacheConstValue.PROCACHECONF, maxSize));
			config.setOutPolicy(CachePropertiesUtils.getValue(CacheConstValue.PROCACHECONF, outPolicyType));
			config.setUpdateCacheInterval(CachePropertiesUtils.getLong(CacheConstValue.PROCACHECONF, updateInterval));
			config.setCacheType(CachePropertiesUtils.getValue(CacheConstValue.PROCACHECONF, type));
			config.setLiveTime(CachePropertiesUtils.getLong(CacheConstValue.PROCACHECONF, liveTime));
			configs.put(cacheName, config);
		}
		return config;
	}

	@Override
	public ArrayList<HolmosCacheConfig> parserAll() {
		for(String cacheName:cacheNames)
			parser(cacheName);
		return (ArrayList<HolmosCacheConfig>) configs.values();
	}

}
