package com.holmos.cache.config.parser;

import java.util.ArrayList;
import java.util.Iterator;

import org.dom4j.Document;
import org.dom4j.Element;

import com.holmos.cache.config.HolmosCacheConfig;
import com.holmos.cache.exception.HolmosCacheException;
import com.holmos.cache.utils.CacheBaseUtils;
/**
 * Watsen 平台缓存系统Properties格式配置文件解析器
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * 
 * */
public class XMLParser extends AbstractParser{
	
	@Override
	public HolmosCacheConfig parser(String cacheName) {
		HolmosCacheConfig config=configs.get(cacheName);
		if(null==config){
			Element cache=getCache(cacheName);
			if(null==cache)
				throw new HolmosCacheException("配置文件中没有名字为"+name+"的缓存信息!");
			config=getConfig(cache);
		}
		return config;
	}

	@SuppressWarnings("unchecked")
	@Override
	public ArrayList<HolmosCacheConfig> parserAll() {
		Document document=CacheBaseUtils.getDocument(System.getenv("user.dir")+"\\conf\\cacheconf.xml");
		Iterator<Element> rootIt=document.getRootElement().elementIterator();
		while(rootIt.hasNext()){
			Element cache=rootIt.next();
			HolmosCacheConfig config=getConfig(cache);
			configs.put(config.getName(), config);
		}return (ArrayList<HolmosCacheConfig>) configs.values();
	}
	
	/**
	 * 寻找匹配的<cache>节点
	 * */
	@SuppressWarnings("unchecked")
	private static Element getCache(String cacheName){
		Document document=CacheBaseUtils.getDocument(System.getenv("user.dir")+"\\conf\\cacheconf.xml");
		Iterator<Element> rootIt=document.getRootElement().elementIterator();
		while(rootIt.hasNext()){
			Element cache=rootIt.next();
			if(match(cache,cacheName)){
				return cache;
			}	
		}return null;
	}

	/**
	 * 判断<cache>节点的name是否和给定的cacheName匹配，如果匹配则返回true，否则返回false
	 * */
	@SuppressWarnings("unchecked")
	private static boolean match(Element cache, String cacheName) {
		Iterator<Element>it=cache.elementIterator();
		while(it.hasNext()){
			Element attribute=it.next();
			if(attribute.getName().equalsIgnoreCase("name")){
				if(attribute.getTextTrim().equalsIgnoreCase(cacheName))
					return true;
			}
		}
		return false;
	}
	
	private static HolmosCacheConfig getConfig(Element cache){
		HolmosCacheConfig config=new HolmosCacheConfig();
		config.setName(getValue(cache, name));
		config.setCacheType(getValue(cache, type));
		config.setMaxSize(Long.parseLong(getValue(cache, maxSize)));
		config.setOutPolicy(getValue(cache, outPolicyType));
		config.setUpdateCacheInterval(Long.parseLong(getValue(cache, updateInterval)));
		config.setLiveTime(Long.parseLong(getValue(cache, liveTime)));
		return config;
	}
	@SuppressWarnings("unchecked")
	private static String getValue(Element element,String name){
		Iterator<Element> it=element.elementIterator();
		while(it.hasNext()){
			Element node=it.next();
			if(node.getName().equalsIgnoreCase(name))
				return node.getTextTrim();
		}
		return null;
	}
}
