package com.holmos.cache.data;

import java.io.Serializable;

/**
 * Wasen 平台缓存系统用到的缓存数据的数据结构
 * 
 * */
public final class HolmosCacheData implements Serializable{

	private static final long serialVersionUID = 1L;

	/**缓存索引*/
	private Object key;
	/**缓存对象的值*/
	private Object value;
	/**元素创建时间*/
	private long createdTime;
	/**最后一次被访问的时间*/
	private long lastVisitTime;
	/**最后一次被更新的时间*/
	private long lastUpdateTime;
	/**被访问的次数*/
	private long visitTimes;
	/**更新次数*/
	private long updateTimes;
	/**空闲时间*/
	private long idleTime;
	/**超时时间*/
	private long expireTime; 
	/**用来指示是否过期*/
	private boolean expire;
	/**指示是否是新增的*/
	private boolean isAdded;
	/**指示是否变更的*/
	private boolean isUpdated;
	/**指示是否是删除的*/
	private boolean isDeleted;
	/**数据是否被锁住*/
	private boolean isLocked;
	/**该数据所占用内存大小*/
	private int size;
	
	private HolmosCacheData(Object key,Object value){
		this.key=key;this.setValue(value);
		this.lastUpdateTime=System.currentTimeMillis();
		this.lastVisitTime=this.lastUpdateTime;
		this.expire=false;
	}
	
	/**
	 * 创建新的缓存数据对象
	 * 
	 * @param key 缓存对象在缓存中的索引
	 * @param value 缓存对象的值
	 * */
	public static HolmosCacheData createCacheData(Object key,Object value){
		return new HolmosCacheData(key, value);
	}
	
	/**
	 * 判断此缓存对象是否已经过期
	 * 
	 * <li>如果expire指示的已经过期，那么直接返回</li>
	 * <li>从最后一次访问到现在，如果时间间隔大于空闲时间，则失效过期</li>
	 * */
	public boolean isExpire() {
		if(!expire){
			long now=System.currentTimeMillis();
			if(now-lastVisitTime>idleTime)
				expire=true;
		}
		return this.expire;
	}

	public void setExpire(boolean expire) {
		this.expire = expire;
	}
	
	public Object getKey() {
		return key;
	}

	public long getLastVisitTime() {
		return lastVisitTime;
	}

	public long getLastUpdateTime() {
		return lastUpdateTime;
	}

	public long getVisitTimes() {
		return visitTimes;
	}

	public long getUpdateTimes() {
		return updateTimes;
	}

	public void setIdleTime(long idleTime){
		this.idleTime=idleTime;
	}

	public Object getValue() {
		return value;
	}

	public void setValue(Object value) {
		this.value = value;
	}

	public boolean isAdded() {
		return isAdded;
	}

	public void setAdded(boolean isAdded) {
		this.isAdded = isAdded;
	}

	public boolean isUpdated() {
		return isUpdated;
	}

	public void setUpdated(boolean isUpdated) {
		this.isUpdated = isUpdated;
	}

	public boolean isDeleted() {
		return isDeleted;
	}

	public void setDeleted(boolean isDeleted) {
		this.isDeleted = isDeleted;
	}

	public long getExpireTime() {
		return expireTime;
	}

	public void setExpireTime(long expireTime) {
		this.expireTime = expireTime;
	}

	public boolean isLocked() {
		return isLocked;
	}

	public void setLocked(boolean isLocked) {
		this.isLocked = isLocked;
	}
	
	/**
	 * 访问后，更新访问状态
	 * */
	public void visited(){
		visitTimes++;
		lastVisitTime=System.currentTimeMillis();
	}
	public void updated(){
		updateTimes++;
		lastUpdateTime=System.currentTimeMillis();
		visited();
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public long getCreatedTime() {
		return createdTime;
	}

	public void setCreatedTime(long createdTime) {
		this.createdTime = createdTime;
	}
}
