package com.holmos.cache.data;

import java.util.HashMap;
import java.util.Map;
/**
 * 查询数据的时候的状态信息
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class HolmosResultCode {
	
	public static final Map<Integer, HolmosResultCode> codeMap = new HashMap<Integer, HolmosResultCode>();
	
	public static final HolmosResultCode SUCCESS = createHolmosResultCode(0, "success");
	public static final HolmosResultCode DATANOTEXSITS = createHolmosResultCode(1, "data not exist");
	public static final HolmosResultCode CONNERROR = createHolmosResultCode(-1, "connection error or timeout");
	public static final HolmosResultCode UNKNOW = createHolmosResultCode(-2, "unkonw error");
	public static final HolmosResultCode TYPENOTMATCH = createHolmosResultCode(-3, "type not match");
	public static final HolmosResultCode ITEMEMPRY = createHolmosResultCode(-4, "item empty");
	public static final HolmosResultCode OUTOFRANGE = createHolmosResultCode(-5, "item out of range");
	public static final HolmosResultCode ITEMSIZEERROR = createHolmosResultCode(-6, "item size error");
	public static final HolmosResultCode SENDFAILED = createHolmosResultCode(-7, "send packet failed");
	public static final HolmosResultCode TIMEOUT = createHolmosResultCode(-8, "timeout");
	public static final HolmosResultCode DATAEXPIRED = createHolmosResultCode(-9, "data expired");
	public static final HolmosResultCode SERVERCANTWORK = createHolmosResultCode(-10, "server can not work");
	public static final HolmosResultCode INVALIDARG = createHolmosResultCode(-11, "invalid argument");
	public static final HolmosResultCode CANNT_OVERRIDE = createHolmosResultCode(-12, "can not override");
	public static final HolmosResultCode KEYTOLARGE = createHolmosResultCode(-13, "key length error");
	public static final HolmosResultCode VALUETOLARGE = createHolmosResultCode(-14, "value length error");
	public static final HolmosResultCode PARTSUCC = createHolmosResultCode(-15, "partly success");
	public static final HolmosResultCode COUNT_BOUNDS = createHolmosResultCode(-16, "count reach bound");
	public static final HolmosResultCode COUNT_ZERO = createHolmosResultCode(-17, "count reach zero");
	public static final HolmosResultCode COUNT_NOTFOUND = createHolmosResultCode(-18, "count not exist");
	
	public static final HolmosResultCode LOCKED= new HolmosResultCode(-29, "data is locked");
	public static final HolmosResultCode LOCK_EXIST = new HolmosResultCode(-19, "lock already exist");
	public static final HolmosResultCode LOCK_NOT_EXIST = new HolmosResultCode(-20, "lock not exist");

	public static final HolmosResultCode TARGET_NOT_NUMBER = createHolmosResultCode(-21, "target not number");
	public static final HolmosResultCode TARGET_NOT_INTEGER = createHolmosResultCode(-22, "target not integer");
	public static final HolmosResultCode TARGET_NOT_DOUBLE = createHolmosResultCode(-23, "target not double");
	public static final HolmosResultCode TARGET_ALREADY_EXIST = createHolmosResultCode(-24, "target has exist");
	public static final HolmosResultCode TARGET_RANGE_HAVE_NONE = createHolmosResultCode(-25, "range_have none");
	public static final HolmosResultCode TAIR_INCDECR_OVERFLOW = createHolmosResultCode(-26, "data overflow");
	public static final HolmosResultCode TAIR_DATA_LEN_LIMIT = createHolmosResultCode(-27, "data len too long");
	
	public static final HolmosResultCode TAIR_IS_NOT_SUPPORT = createHolmosResultCode(-28, "not support");
	
	private int code = 0;
	private String message = null;

	private HolmosResultCode(int code) {
		this.code = code;
	}

	private HolmosResultCode(int code, String message) {
		this.code = code;
		this.message = message;
	}

	private static HolmosResultCode createHolmosResultCode(int code, String message) {
		HolmosResultCode holmosResultCode = new HolmosResultCode(code, message);
		if(code == 1) {
			codeMap.put(-3998, holmosResultCode);
		}
		codeMap.put(code, holmosResultCode);
		return holmosResultCode;
	}
	
	public static HolmosResultCode valueOf(int code) {
		HolmosResultCode HolmosResultCode = codeMap.get(code);
		if(HolmosResultCode == null) {
			return UNKNOW;
		}
		return HolmosResultCode;
	}
	
	public int getCode() {
		return code;
	}

	public String getMessage() {
		return message;
	}
	
	
	public boolean isSuccess() {
		return code >= 0;
	}

	@Override
	public String toString() {
		return "code=" + code + ", msg=" + message;
	}

	@Override
	public boolean equals(Object obj) {
		if(obj != null && (obj instanceof HolmosResultCode)) {
			HolmosResultCode rc = (HolmosResultCode)obj;
			return rc.getCode() == code;
		}
		return false;
	}

	@Override
	public int hashCode() {
		return code;
	}
}
