package com.holmos.cache.outpolicy;

import com.holmos.cache.exception.HolmosCacheException;
import com.holmos.cache.store.HolmosCacheStore;

/**
 * Watsen 平台缓存系统数据替换策略工厂方法
 * 
 * @author 吴银龙()
 * */
public class HolmosCacheOutPolicyFactory {
	public static HolmosFIFOOutPolicy createFIFOOutPolicy(HolmosCacheStore cacheStore){
		return new HolmosFIFOOutPolicy(cacheStore);
	}
	public static HolmosLFUOutPolicy createLFUOutPolicy(HolmosCacheStore cacheStore){
		return new HolmosLFUOutPolicy(cacheStore);
	}
	public static HolmosLRUOutPolicy createLRUOutPolicy(HolmosCacheStore cacheStore){
		return new HolmosLRUOutPolicy(cacheStore);
	}
	public static HolmosAbstractCacheOutPolicy createCacheOutPolicy(HolmosDataOutPolicy outPolicy,HolmosCacheStore cacheStore){
		switch (outPolicy) {
		case FIFO:
			return createFIFOOutPolicy(cacheStore);
		case LFU:
			return createLFUOutPolicy(cacheStore);
		case LRU:
			return createLFUOutPolicy(cacheStore);
		default:
			return null;
		}
	}
	public static HolmosAbstractCacheOutPolicy createCacheOutPolicy(String outPolicyClassName){
		for(String outPolicyFullClassName:
			HolmosCacheOutPolicyManager.getInstance().getAllFullOutPolicyClassNames()){
			if(outPolicyFullClassName.endsWith(outPolicyClassName))
				try {
					return (HolmosAbstractCacheOutPolicy) Class.forName(outPolicyFullClassName).newInstance();
				} catch (Exception e){
					throw new HolmosCacheException("创建替换策略失败!请查看所给的策略在定义的时候对" +
							"HolmosCacheOutPolicyManager 的操作是否正确!");
				}
		}return null;
	}
}
