package com.holmos.cache.size.layout.impl;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import com.holmos.cache.size.layout.ClassSizeInfoParser;
import com.holmos.cache.size.layout.MemoryLayoutSpecification;

/**
 * sun 公司的hotspot 32位JVM的对象内部字段占用内存布局实现
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class Hotspot32MemoryLayoutSpecification implements MemoryLayoutSpecification{

	@Override
	public int getArrayHeaderSize() {
		return 12;
	}

	@Override
	public int getObjectHeaderSize() {
		return 8;
	}

	@Override
	public int getObjectPadding() {
		return 8;
	}

	@Override
	public int getReferenceSize() {
		return 4;
	}

	@Override
	public int outerClassReferenceSize() {
		return 4;
	}
	
	@SuppressWarnings({ "rawtypes" })
	@Override
	public int getCommonFieldsSize(Class clazz) {
		int[] fieldLayOutInfo=new int[5];
		for(Field field : clazz.getDeclaredFields()){
			if((field.getModifiers()&Modifier.STATIC)==0)
				fieldLayOutInfo[ClassSizeInfoParser.getPrimitiveType(field.getType())]++;
		}
		return getSize(fieldLayOutInfo);
	}
	
	private int getSize(int[] fieldLayOutInfo) {
		int eightCount=0;
		fieldLayOutInfo[1]+=fieldLayOutInfo[0]/2;
		fieldLayOutInfo[2]+=fieldLayOutInfo[1]/2;
		fieldLayOutInfo[3]+=fieldLayOutInfo[2]/2;
		eightCount+=fieldLayOutInfo[3];
		fieldLayOutInfo[0]%=2;
		fieldLayOutInfo[1]%=2;
		fieldLayOutInfo[2]%=2;
		int remain=8-fieldLayOutInfo[0]-fieldLayOutInfo[1]-fieldLayOutInfo[2];
		if(remain!=8)
			eightCount++;
		if(remain>=4)
			if(fieldLayOutInfo[4]>1)
				fieldLayOutInfo[4]--;
		eightCount+=(fieldLayOutInfo[4]/2+fieldLayOutInfo[2]%2);
		return eightCount*8;
	}

	@Override
	public int getDataSizeOnce() {
		return 8;
	}

	@Override
	public int getPrimitiveSize(Class<?> type) {
		if(long.class == type || double.class == type)
			return 8;
		else if(int.class == type || float.class == type)
			return 4;
		else if(char.class == type || short.class == type)
			return 2;
		else if(boolean.class == type || byte.class == type)
			return 1;
		else
			return 4;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public int getStaticFieldsSize(Class clazz) {
		int[] fieldLayOutInfo=new int[5];
		for(Field field : clazz.getDeclaredFields()){
			if((field.getModifiers()&Modifier.STATIC)!=0)
				fieldLayOutInfo[ClassSizeInfoParser.getPrimitiveType(field.getType())]++;
		}
		return getSize(fieldLayOutInfo);
	}
	
}
