package com.holmos.cache.state;

import java.io.Serializable;
/**
 * Watsen 平台缓存状态信息，根据此状态信息可以获取和描述当前缓存的使用情况
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * 
 * */
public class HolmosCacheState implements Serializable{

	private static final long serialVersionUID = 1L;
	
	/**缓存的层级*/
	private byte level;
	
	/**缓存元素个数*/
	private long dataMount;
	
	/**缓存大小，单位Byte*/
	private long size;
	
	/**缓存读取次数*/
	private long visitTimes;

	/**缓存更新次数，指对以前缓存中已有数据的更新次数*/
	private long updateTimes;
	
	/**缓存写入次数，指原来在缓存中没有的数据*/
	private long putTimes;
	
	/**缓存数据的存活时间*/
	private long liveTime;
	
	/**命中次数*/
	private long hitTime;
	
	/**没有命中的次数*/
	private long missTime;
	
	public byte getLevel() {
		return level;
	}

	public long getDataMount() {
		return dataMount;
	}
	
	/**加入元素之后的操作*/
	public void addDataMount(int mount){
		dataMount+=mount;
	}
	
	public long getSize() {
		return size;
	}
	
	/**增加一个元素的或者更新一个元素的时候的操作*/
	public void addSize(int size){
		this.size+=size;
	}
	
	public long getVisitTimes() {
		return visitTimes;
	}

	/**访问元素之后*/
	public void addVisitTimes(int visitTimes){
		this.visitTimes+=visitTimes;
	}
	
	public long getPutTimes() {
		return putTimes;
	}

	/**元素放入缓存之后*/
	public void addPutTimes(int putTimes){
		this.putTimes+=putTimes;
	}
	
	public long getUpdateTimes() {
		return updateTimes;
	}

	/**缓存元素更新之后*/
	public void addUpdateTimes(int updateTimes){
		this.updateTimes+=updateTimes;
	}
	
	public HolmosCacheState(){
		this.dataMount=this.size=0;
		this.putTimes=this.visitTimes=this.updateTimes=0;
	}

	public long getLiveTime() {
		return liveTime;
	}

	public void setLiveTime(long liveTime) {
		this.liveTime = liveTime;
	}

	public long getMissTime() {
		return missTime;
	}

	public void addMissTime(long missTime) {
		this.missTime++;
	}

	public long addHitTime() {
		return hitTime;
	}

	public void addHitTime(long hitTime) {
		this.hitTime++;
	}
}
