package com.holmos.cache.store;

/**
 * Watsen 平台缓存仓库的工厂方法
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class HolmosCacheStoreFactory {
	
	/**
	 * 新建一个内存缓存仓库
	 * 
	 * <li>此仓库中没有数据</li>
	 * 
	 * @return 返回新建的内存缓存仓库
	 * */
	public static HolmosMemoryCacheStore createMemoryCacheStore(){
		HolmosMemoryCacheStore cacheStore=new HolmosMemoryCacheStore();
		
		return cacheStore;
	}
	
	/**
	 * 新建一个硬盘缓存仓库
	 * 
	 * <li>此仓库中没有数据</li>
	 * 
	 * @return 返回新建的硬盘缓存仓库
	 * */
	public static HolmosDiskCacheStore createDiskCacheStore(){
		HolmosDiskCacheStore cacheStore=new HolmosDiskCacheStore();
		
		return cacheStore;
	}
	
	/**
	 * 根据指定要求的数据仓库类型来新建数据仓库
	 * 
	 * @param cacheStoreType 指定要求的数据仓库类型
	 * @return 新建的数据仓库
	 * */
	public static HolmosCacheStore createCacheStore(HolmosCacheStoreType cacheStoreType){
		switch (cacheStoreType) {
		case MEMORY:
			return createMemoryCacheStore();
		case DISK:
			return createDiskCacheStore();
		}return null;
	}
}
